/*
 * biteMe.c
 * 
 * create a basic /proc file entry
 * by Mark Claypool, 2001
 * (modified from procfs.c, by Ori Pomerantz)
 * 
 * compile with: gcc -c -D__KERNEL__ -DMODULE biteMe.c
 */ 

/* include files */
#include <linux/kernel.h>	/* kernel stuff */
#include <linux/module.h>	/* module stuff */
#include <linux/proc_fs.h>	/* /proc stuff */

/* function prototypes */
int procfile_read(char *buff, char **buff_out, off_t offset, int buff_len, 
		  int zero);

/* global (to this module) variables */
struct proc_dir_entry myProcFile = {
    0,              /* inode number - (filled in by proc_register[_dynamic]) */
    6,              /* length of the file name */
    "biteMe",      /* file name under /proc */
    S_IFREG | S_IRUGO, /* file permissions (regular file, -r--r--r--) */
    1,	            /* num of dirs where the file is referenced (links) */
    0, 0,           /* uid, gid for the file (root,root) */
    100,            /* file size (try /bin/ll -l) */
    NULL,           /* allowed functions (linking, removing ....) none, here */
    procfile_read,  /* called when file is read */
    NULL /* misc for setting file's inode, perms, ownership ... */
}; 

/* Read out data from the proc file.

   INPUT:
   buff: The buffer where the data is to be inserted, if 
      you decide to use it.
   buff_out: A pointer to a pointer to characters. This is 
      useful if you don't want to use the buffer 
      allocated by the kernel.
   offset: the current position in the file. 
   buff_len: the size of the buffer in the first argument.  
   zero: not used (possibly, for future use)

   DO:
   Copy the data into buff, being careful not to go past the length
   allowed.  Or, if the buffer is stored in the module itself (like in
   this example, it should be declared static.  In this case, put its
   location in the second argument.
 
   RETURN:
   Return the number of bytes in the buffer.  Zero means there is no
   further information at this time (end of file). A negative return
   value is an error condition.
   
 */

int procfile_read(char *buff, char **buff_out, off_t offset, int buff_len, 
		  int zero) {
  int len;			/* number of bytes actually used */
  char suffix[3];		/* for the number ending */
  static char my_buff[80];	/* static so stays in memory */
  static int num_reads = 0;	/* stores number of times read */

  /* 
   * The standard library read call issues the read system call 
   * until there is no more information returned or until the
   * buffer is filled.  Since all our information comes out
   * in one read call, we return indicating no more information.
   */
  if (offset > 0)
    return 0;

  num_reads++;

  /* fill the buffer and get its length */
  if (num_reads % 100 > 10 && num_reads % 100 < 14)
    sprintf(suffix, "th");
  else if (num_reads % 10 == 1) 
    sprintf(suffix, "st");
  else if (num_reads % 10 == 2)
    sprintf(suffix, "nd");
  else if (num_reads % 10 == 3)
    sprintf(suffix, "rd");
  else
    sprintf(suffix, "th");

  /* Make the string and tell where it is */
  len = sprintf(my_buff, "For the %d%s time ... Bite me!\n", num_reads, 
		suffix);
  *buff_out = my_buff;

  /* return the length (standard for read calls) */
  return len;
}

/* init module and register with /proc fs */
int init_module() {
  extern struct proc_dir_entry proc_root; /* /proc (in linux/fs/proc/root.c) */
  return proc_register(&proc_root, &myProcFile);
}


/* cleanup module and unregister from /proc fs */
void cleanup_module() {
  extern struct proc_dir_entry proc_root; /* /proc (in linux/fs/proc/root.c) */
  proc_unregister(&proc_root, myProcFile.low_ino);
}
